#ifndef _CHRONOPTICS_TOF_USB_INTERFACE_H_
#define _CHRONOPTICS_TOF_USB_INTERFACE_H_

#include <chronoptics/tof/camera_config.h>
#include <chronoptics/tof/calibration.h>
#include <chronoptics/tof/data.h>
#include <chronoptics/tof/gige_interface.h>

#ifdef __cplusplus
extern "C" {
#endif

/** USB device
 */
typedef struct tof_usb_device* tof_usb_device_t;

/** Destruct tof_usb_device */
TOF_EXPORT void tof_usb_device_delete(tof_usb_device_t ptr);

/** Get serial corresponding with USB device
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Serial
 */
TOF_EXPORT const char* tof_usb_device_serial(tof_usb_device_t ptr, tof_error_t *error);

/** Connect to a camera directly using the USB interface
 */
typedef struct tof_usb_interface* tof_usb_interface_t;

/** Destruct tof_usb_interface */
TOF_EXPORT void tof_usb_interface_delete(tof_usb_interface_t ptr);

/** Create USB camera from a usb device
 * @param ptr Pointer to class
 * @param usb_device USB device
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_usb_interface_t tof_usb_interface_new(tof_usb_device_t usb_device, tof_error_t *error);

/** Check whether a connection is still maintained with the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Is connected
 */
TOF_EXPORT bool tof_usb_interface_is_connected(const tof_usb_interface_t ptr, tof_error_t *error);

/** Tests the connection speed of the USB port. This is useful to check whether
 * a hub or USB port works well. The maximum speed that the camera is able to
 * transmit is ~3.0 Gbps.
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Connection speed in Gpbs (bits)
 */
TOF_EXPORT double tof_usb_interface_test_stream_speed(tof_usb_interface_t ptr, tof_error_t *error);

/** Download camera configuration from the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Camera configuration
 */
TOF_EXPORT tof_camera_config_t tof_usb_interface_download_configuration(tof_usb_interface_t ptr, tof_error_t *error);

/** Download calibration from the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Calibration
 */
TOF_EXPORT tof_calibration_t tof_usb_interface_download_calibration(tof_usb_interface_t ptr, tof_error_t *error);

/** Upload camera configuration to the camera
 * @param ptr Pointer to class
 * @param config Camera configuration
 * @param error Pointer to error
 */
TOF_EXPORT void tof_usb_interface_upload_configuration(tof_usb_interface_t ptr, const tof_camera_config_t config, tof_error_t *error);

/** Check whether this camera is capable of streaming the specified type
 * @param ptr Pointer to class
 * @param stream Depth stream type
 * @param error Pointer to error
 * @return Is capable
 */
TOF_EXPORT bool tof_usb_interface_depth_stream_capable(const tof_usb_interface_t ptr, enum tof_depth_stream_type stream, tof_error_t *error);

/** Start the depth stream
 * @param ptr Pointer to class
 * @param stream Depth stream type
 * @param error Pointer to error
 */
TOF_EXPORT void tof_usb_interface_start_depth_stream(tof_usb_interface_t ptr, enum tof_depth_stream_type stream, tof_error_t *error);

/** Stop the depth stream
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_usb_interface_stop_depth_stream(tof_usb_interface_t ptr, tof_error_t *error);

/** Returns true if a depth frame is available
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Depth frame available
 */
TOF_EXPORT bool tof_usb_interface_has_depth_data(const tof_usb_interface_t ptr, tof_error_t *error);

/** Get a depth frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Depth frame
 */
TOF_EXPORT tof_data_t tof_usb_interface_get_depth_data(tof_usb_interface_t ptr, tof_error_t *error);

/** Check whether camera supports an image stream
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Camera supports an image stream
 */
TOF_EXPORT bool tof_usb_interface_image_stream_capable(const tof_usb_interface_t ptr, tof_error_t *error);

/** Start image stream
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_usb_interface_start_image_stream(tof_usb_interface_t ptr, tof_error_t *error);

/** Stop image stream
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_usb_interface_stop_image_stream(tof_usb_interface_t ptr, tof_error_t *error);

/** Check whether image frame is available
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Image frame available
 */
TOF_EXPORT bool tof_usb_interface_has_image_data(const tof_usb_interface_t ptr, tof_error_t *error);

/** Get image frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Image data
 */
TOF_EXPORT tof_data_t tof_usb_interface_get_image_data(tof_usb_interface_t ptr, tof_error_t *error);

/** Get the image frame into the supplied pointer
 * @param ptr Pointer to class
 * @param pointer The pointer to the data to write into
 * @param capacity The amount of data the pointer can hold
 * @param callback Callback that will be called when pointer is no longer in use
 * @param callback_user_data User data that will be passed back when the function pointer is called
 * @param error Pointer to error
 * @return Image data
 */
TOF_EXPORT tof_data_t tof_usb_interface_get_image_data_into_pointer(tof_usb_interface_t ptr, uint8_t* pointer, size_t capacity, tof_user_pointer_destructed_t callback, void* callback_user_data, tof_error_t *error);

/** Get the capacity for amount of user pointers that can be registered
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return User pointer capacity
 */
TOF_EXPORT size_t tof_usb_interface_get_user_pointer_capacity(const tof_usb_interface_t ptr, tof_error_t *error);

/** Set the capacity for amount of user pointers that can be registered
 * @param ptr Pointer to class
 * @param capacity User pointer capacity
 * @param error Pointer to error
 */
TOF_EXPORT void tof_usb_interface_set_user_pointer_capacity(tof_usb_interface_t ptr, size_t capacity, tof_error_t *error);

/** Add a pointer that will be filled with depth data. This method allows you to
 * fill data straight into your own data structure without any additional
 * copying.
 * @param ptr Pointer to class
 * @param pointer The pointer to the data to write into
 * @param capacity The amount of data the pointer can hold
 * @param callback Callback that will be called when pointer is no longer in use
 * @param callback_user_data User data that will be passed back when the function pointer is called
 * @param error Pointer to error
 */
TOF_EXPORT void tof_usb_interface_add_depth_user_pointer(tof_usb_interface_t ptr, uint8_t* pointer, size_t capacity, tof_user_pointer_destructed_t callback, void* callback_user_data, tof_error_t *error);

/** Get the tof library version running on the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Version
 */
TOF_EXPORT const char* tof_usb_interface_version(tof_usb_interface_t ptr, tof_error_t *error);

/** Check whether the camera is capable of software trigger
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Software trigger capable
 */
TOF_EXPORT bool tof_usb_interface_software_trigger_capable(tof_usb_interface_t ptr, tof_error_t *error);

/** Software trigger the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 */
TOF_EXPORT void tof_usb_interface_software_trigger(tof_usb_interface_t ptr, tof_error_t *error);

/** Discover all connected chronoptics usb cameras
 * @param usb_devices The usb devices
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_usb_device_discover(tof_usb_device_t* usb_devices, size_t capacity, tof_error_t *error);

/** Discover one chronoptics usb camera
 * @param error Pointer to error
 * @return A USB device
 */
TOF_EXPORT tof_usb_device_t tof_usb_device_discover_one(tof_error_t *error);

/** Discover usb camera with the given serial
 * @param serial Serial of the camera to find
 * @param error Pointer to error
 * @return A USB device
 */
TOF_EXPORT tof_usb_device_t tof_usb_device_find(const char* serial, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
