classdef Calibration < handle
  % Calibration
  %   This class contains all the calibration information
  %
  % Methods:
  %   write - Write calibration to disk
  %   getCalibratedFrequencies - The the calibrated frequencies in the
  %                              calibration file
  %   getFocalLengthX - Get focal length x
  %   getFocalLengthY - Get focal length y
  %   getPrincipalPointX - Get principal point x
  %   getPrincipalPointY - Get principal point y
  %   getCameraSerial - Get the serial of the camera for which this calibration
  %                     file works
  %   setCameraSerial - Set the serial of the camera for which this calibration
  %                     file works
  %   getDepthCameraMatrix - Get the camera matrix of the depth sensor and lens
  %   setDepthCameraMatrix - Set the camera matrix of the depth sensor and lens
  %   getDepthDistortionCoefficients - Get the distortion coefficients of the
  %                                    depth lens
  %   setDepthDistortionCoefficients - Set the distortion coefficients of the
  %                                    depth lens
  %   getTrustRadius - Get the trust radius of the lens calibration
  %   setTrustRadius - Set the trust radius of the lens calibration
  %   getRgbCameraMatrix - Get the camera matrix of the rgb sensor and lens
  %   setRgbCameraMatrix - Set the camera matrix of the rgb sensor and lens
  %   getRgbDistortionCoefficients - Get the distortion coefficients of the rgb
  %                                  lens
  %   setRgbDistortionCoefficients - the distortion coefficients of the rgb lens
  %   getRotationVector - Get the rotation vector between the depth and rgb
  %                       sensor
  %   setRotationVector - Set the rotation vector between the depth and rgb
  %                       sensor
  %   getTranslationVector - Get the translation vector between the depth and
  %                          rgb sensor
  %   setTranslationVector - Set the translation vector between the depth and
  %                          rgb sensor
  %   getBadPixRows - Get the number of rows that are calibrated
  %   setBadPixRows - Set the number of rows that are calibrated
  %   getBadPixCols - Get the number of columns that are calibrated
  %   setBadPixCols - Set the number of columns that are calibrated
  %   getBadPixels - Get the bad pixels on the depth sensor. Every coordinate in
  %                  the vector is one bad pixel
  %   setBadPixels - Set the bad pixels on the depth sensor. Every coordinate in
  %                  the vector is one bad pixel
  %   getFpos - Get the fixed phase and amplitude calibration
  %   setFpos - Set the fixed phase and amplitude calibration
  %   getRawCals - Get the raw offsets
  %   setRawCals - Set the raw offsets
  %   getRawCmOffsets - Get the raw common offsets
  %   setRawCmOffsets - Set the raw common offsets
  %   setDotMask - Set the dot mask. The dot mask is specific for dot
  %                projectors. Each bit in the mask corresponds to a certain
  %                distance range, the distance is specified in the ranges as
  %                follows: 0 - range[0], range[0] - range[1] ... range[N] - inf
  %   hasDotMask - Check if the dot mask is set
  %   clearDotMask - Clear dot mask if set
  %   getDotMaskMask - Get the mask
  %   getDotMastRanges - Get the ranges
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = Calibration(varargin)
      % Calibration
      %   Construct new calibration class
      %
      % ----------
      %
      %   Create default calibration
      %
      % ----------
      %
      %   Read calibration from disk
      %
      % Inputs:
      %   1 - string - Location of calibration file
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      elseif nargin == 1 && (isStringScalar(varargin{1}) || ischar(varargin{1}))
        obj.mPtr = tof.mex('tof_calibration_new_from_disk', varargin{1});
      elseif nargin == 0 && (isStringScalar(varargin{1}) || ischar(varargin{1}))
        obj.mPtr = tof.mex('tof_calibration_new_from_disk', );
      else
        error("Can't construct empty object");
      end
    end


    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_calibration_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function write(obj, file_location)
      % write
      %   Write calibration to disk
      %
      % Inputs:
      %   file_location - string - Location to save the calibration file to
      %

      tof.mex('tof_calibration_write', obj.mPtr, file_location);
    end

    function ret_val = getCalibratedFrequencies(obj)
      % getCalibratedFrequencies
      %   The the calibrated frequencies in the calibration file
      %
      % Output:
      %   ret_val - vector[numeric] - The calibrated modulation frequencies
      %

      ret_val = tof.mex('tof_calibration_get_calibrated_frequencies', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getFocalLengthX(obj)
      % getFocalLengthX
      %   Get focal length x
      %
      % Output:
      %   ret_val - numeric - Focal length
      %

      ret_val = tof.mex('tof_calibration_get_focal_length_x', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getFocalLengthY(obj)
      % getFocalLengthY
      %   Get focal length y
      %
      % Output:
      %   ret_val - numeric - Focal length
      %

      ret_val = tof.mex('tof_calibration_get_focal_length_y', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getPrincipalPointX(obj)
      % getPrincipalPointX
      %   Get principal point x
      %
      % Output:
      %   ret_val - numeric - Principal point
      %

      ret_val = tof.mex('tof_calibration_get_principal_point_x', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getPrincipalPointY(obj)
      % getPrincipalPointY
      %   Get principal point y
      %
      % Output:
      %   ret_val - numeric - Principal point
      %

      ret_val = tof.mex('tof_calibration_get_principal_point_y', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getDepthCameraMatrix(obj)
      % getDepthCameraMatrix
      %   Get the camera matrix of the depth sensor and lens
      %
      % Output:
      %   ret_val - array[numeric] - Camera matrix, 3x3 matrix
      %

      ret_val = tof.mex('tof_calibration_get_depth_camera_matrix', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getDepthDistortionCoefficients(obj)
      % getDepthDistortionCoefficients
      %   Get the distortion coefficients of the depth lens
      %
      % Output:
      %   ret_val - array[numeric] - Distortion coefficients, 5x1 matrix
      %

      ret_val = tof.mex('tof_calibration_get_depth_distortion_coefficients', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getRgbCameraMatrix(obj)
      % getRgbCameraMatrix
      %   Get the camera matrix of the rgb sensor and lens
      %
      % Output:
      %   ret_val - array[numeric] - Camera matrix, 3x3 matrix
      %

      ret_val = tof.mex('tof_calibration_get_rgb_camera_matrix', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getRgbDistortionCoefficients(obj)
      % getRgbDistortionCoefficients
      %   Get the distortion coefficients of the rgb lens
      %
      % Output:
      %   ret_val - array[numeric] - Distortion coefficients, 5x1 matrix
      %

      ret_val = tof.mex('tof_calibration_get_rgb_distortion_coefficients', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getRotationVector(obj)
      % getRotationVector
      %   Get the rotation vector between the depth and rgb sensor
      %
      % Output:
      %   ret_val - array[numeric] - Rotation vector, 3x1 matrix
      %

      ret_val = tof.mex('tof_calibration_get_rotation_vector', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getTranslationVector(obj)
      % getTranslationVector
      %   Get the translation vector between the depth and rgb sensor
      %
      % Output:
      %   ret_val - array[numeric] - Translation vector, 3x1 matrix
      %

      ret_val = tof.mex('tof_calibration_get_translation_vector', obj.mPtr);
      ret_val = double(ret_val);
    end

  end
end
